@@ -10,17 +10,20 @@ from PIL import Image
 
 from numpy import asarray
 
-def get_rgb_from_int(value: int) -> typing.Tuple[int,int,int]:
-    blue =  value & 255
+
+def get_rgb_from_int(value: int) -> typing.Tuple[int, int, int]:
+    blue = value & 255
     green = (value >> 8) & 255
-    red =   (value >> 16) & 255
+    red = (value >> 16) & 255
     return red, green, blue
 
-def get_int_from_rgb(rgb: typing.Tuple[int,int,int]) -> int:
+
+def get_int_from_rgb(rgb: typing.Tuple[int, int, int]) -> int:
     red = rgb[0]
     green = rgb[1]
     blue = rgb[2]
-    return (red<<16) + (green<<8) + blue
+    return (red << 16) + (green << 8) + blue
+
 
 def get_plot_extent(
     self, plot_left, plot_right, plot_bottom, plot_top, slice_value, bb, view_direction
@@ -176,7 +179,7 @@ def get_slice_of_material_ids(
     plot_left: typing.Optional[float] = None,
     plot_right: typing.Optional[float] = None,
     pixels_across: int = 500,
-    verbose: bool = False
+    verbose: bool = False,
 ):
     """Returns a grid of material IDs for each mesh voxel on the slice. This
     can be passed directly to plotting functions like Matplotlib imshow. 0
@@ -279,7 +282,7 @@ def get_slice_of_material_ids(
         my_plot.basis = "xy"
         my_plot.origin = (plot_x, plot_y, slice_value)
     if view_direction == "x":
-        my_plot.basis = "xz"
+        my_plot.basis = "yz"
         my_plot.origin = (slice_value, plot_x, plot_y)
     if view_direction == "y":
         my_plot.basis = "xz"
@@ -322,7 +325,8 @@ def get_slice_of_material_ids(
     #     [inner_entry[0] for inner_entry in outer_entry] for outer_entry in image_values
     # ]
     image_value = [
-        [get_int_from_rgb(inner_entry) for inner_entry in outer_entry] for outer_entry in image_values
+        [get_int_from_rgb(inner_entry) for inner_entry in outer_entry]
+        for outer_entry in image_values
     ]
 
     # replaces rgb (255,255,255) which is 16777215 values with 0.
@@ -394,16 +398,16 @@ def get_slice_of_cell_ids(
         bb = dag_universe.bounding_box
 
     else:
-        
+
         original_materials = self.get_all_materials()
         mat_ids = original_materials.keys()
 
         mat_names = []
         for key, value in original_materials.items():
             mat_names.append(value.name)
-            
+
         bb = self.bounding_box
-    
+
     self.export_to_xml(tmp_folder)
 
     plot_left, plot_right, plot_bottom, plot_top, slice_value = self.get_plot_extent(
@@ -449,7 +453,7 @@ def get_slice_of_cell_ids(
         my_plot.basis = "xy"
         my_plot.origin = (plot_x, plot_y, slice_value)
     if view_direction == "x":
-        my_plot.basis = "xz"
+        my_plot.basis = "yz"
         my_plot.origin = (slice_value, plot_x, plot_y)
     if view_direction == "y":
         my_plot.basis = "xz"
@@ -490,7 +494,8 @@ def get_slice_of_cell_ids(
     # the image_values have three entries for RGB but we just need one.
     # this reduces the nested list to contain a single value per pixel
     image_value = [
-        [get_int_from_rgb(inner_entry) for inner_entry in outer_entry] for outer_entry in image_values
+        [get_int_from_rgb(inner_entry) for inner_entry in outer_entry]
+        for outer_entry in image_values
     ]
 
     # replaces rgb (255,255,255) which is 16777215 values with 0.
